"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScopedServiceProvider = void 0;
var Errors_1 = require("../Errors");
var utils_1 = require("../utils");
var ScopedServiceProvider = /** @class */ (function () {
    function ScopedServiceProvider(parent, lifetime) {
        var _this = this;
        Object.defineProperty(this, "scope", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "singletons", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "lookup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "ordered", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "proxy", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "root", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "parent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_isDone", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        this.parent = parent;
        this.lifetimes = parent.lifetimes;
        this.root = parent.root;
        this.proxy = (0, utils_1.proxyOf)(this);
        this.scope = (parent instanceof ScopedServiceProvider) ? parent.scope : {};
        if (parent instanceof ScopedServiceProvider)
            parent.ordered.forEach(function (e) { return _this._enter(e); });
        if (lifetime)
            this._enter(lifetime);
    }
    Object.defineProperty(ScopedServiceProvider.prototype, "isDone", {
        get: function () {
            return this._isDone;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "depth", {
        get: function () {
            return this.ordered.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "lastSingleton", {
        get: function () {
            return this.singletons[this.singletons.length - 1];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var lifetime = this.getLifetime(selector);
            var context = new ScopedServiceProvider(this, lifetime);
            var result = lifetime.provide(context);
            context._isDone = true;
            return result;
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            if (!lifetime)
                return this;
            return new ScopedServiceProvider(this, lifetime);
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "_enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            if (lifetime.name in this.lookup)
                throw new Errors_1.CircularDependencyError(lifetime.name, this.ordered.map(function (e) { return e.name; }));
            this.ordered.push(lifetime);
            if (lifetime.isSingleton)
                this.singletons.push(lifetime);
            this.lookup[lifetime.name] = lifetime;
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "getLifetime", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = (0, utils_1.extractSelector)(selector);
            var lifetime = this.lifetimes[key];
            if (!lifetime)
                throw new Errors_1.ExistenceDependencyError(key);
            return lifetime;
        }
    });
    return ScopedServiceProvider;
}());
exports.ScopedServiceProvider = ScopedServiceProvider;
